﻿using System.Windows.Controls;

namespace SudokuMachine
{
    class BlenderView
    {
        private MatrixView _matrix;
        public Decorator Chrome { get { return _matrix.Chrome; } }

        private BlenderKeg _keg1;
        private BlenderKeg _keg2;

        public BlenderView(BlenderSlot amountSlot, BowlView bowl1, BowlView bowl2)
        {
            _matrix = new MatrixView(null);

            BlockView[] blocks = _matrix.GetBlocks();

            _keg1 = new BlenderKeg(amountSlot, blocks, bowl1);
            _keg2 = new BlenderKeg(amountSlot, blocks, bowl2);

            _keg1.Couple(_keg2);
        }

        internal void Reset()
        {
            _keg1.Reset();
            _keg2.Reset();
        }
    }

    class BlenderKeg
    {
        private KegBank[] _banks;

        public BlenderKeg(BlenderSlot amountSlot, BlockView[] blocks, BowlView bowl)
        {
            _banks = new KegBank[9];

            for (int i = 0; i < 9; i++)
            {
                _banks[i] = new KegBank(amountSlot, blocks[i]);
            }

            bowl.Bind(_banks);
        }

        internal void Couple(BlenderKeg otherKeg)
        {
            for (int i = 0; i < 9; i++)
            {
                _banks[i].Couple(otherKeg._banks[i]);
            }
        }

        internal void Reset()
        {
            foreach (var bank in _banks)
            {
                bank.Reset();
            }
        }
    }

    class KegBank
    {
        private BlenderSlot _amountSlot;
        private CellView[] _cells;

        private int _index;
        private int _value;

        private KegBank _conjugant;

        public KegBank(BlenderSlot amountSlot, BlockView block)
        {
            _amountSlot = amountSlot;
            _cells = block.GetCells();
        }

        internal void Couple(KegBank otherBank)
        {
            _conjugant = otherBank;
            otherBank._conjugant = this;
        }

        internal void Shift()
        {
            if (++_value > 9) _value = 1;
            if (_index != _conjugant._index) Show();
        }

        internal void Pass(int targetIndex)
        {
            if (targetIndex == _conjugant._index) Merge(targetIndex);
            else
                if (_index == _conjugant._index) Split(targetIndex);
                else
                {
                    Hide();
                    _index = targetIndex;
                    Show();
                }
        }

        private void Hide()
        {
            _cells[_index].Content.Setup(0);
        }

        private void Show()
        {
            _cells[_index].Content.Setup(_value);
        }

        private void Merge(int targetIndex)
        {
            Hide();
            _index = targetIndex;
            _cells[_index].Flash(true);

            _amountSlot.Inc();
        }

        private void Split(int targetIndex)
        {
            _cells[_index].Flash(false);
            _index = targetIndex;
            Show();
            _conjugant.Show();

            _amountSlot.Dec();
        }

        internal void Reset()
        {
            _cells[_index].Flash(false);
        }

        internal void Reset(int index, int value)
        {
            _index = index;
            _value = value;

            _cells[_index].Flash(true);
        }
    }
}