﻿using System;
using System.Windows.Controls;

namespace SudokuMachine
{
    class Dashboard
    {
        private BowlView _bowl1;
        private BowlView _bowl2;
        private BlenderView _blender;

        private DashGun _gun;

        private StackPanel _layout;
        public Panel Layout { get { return _layout; } }

        public Dashboard(StatusBar statusBar)
        {
            var seed = new Random();

            _bowl1 = new BowlView(statusBar.CountSlot1, seed);
            _bowl2 = new BowlView(statusBar.CountSlot2, seed);
            _blender = new BlenderView(statusBar.AmountSlot, _bowl1, _bowl2);

            _gun = new DashGun(_bowl1, _bowl2, seed);

            _layout = new StackPanel { Orientation = Orientation.Horizontal };
            _layout.Children.Add(_bowl1.Chrome);
            _layout.Children.Add(_blender.Chrome);
            _layout.Children.Add(_bowl2.Chrome);
        }

        internal void Kickdown1()
        {
            _bowl1.Bubble();
        }

        internal void Kickdown2()
        {
            _bowl2.Bubble();
        }

        internal void Reset()
        {
            _blender.Reset();
            _bowl1.Reset(2);
            _bowl2.Reset(3);
        }

        internal void Shake()
        {
            _gun.Fire();
        }
    }

    class DashGun
    {
        const int CELL_COUNT = 81 * 2;

        private CellView[] _cells;
        private Random _random;

        public DashGun(BowlView bowl1, BowlView bowl2, Random seed)
        {
            _cells = new CellView[CELL_COUNT];
            _random = new Random(seed.Next());

            var cells1 = bowl1.GetCells();
            var cells2 = bowl2.GetCells();

            int i = 0, j = 0;
            while (i < CELL_COUNT)
            {
                _cells[i++] = cells1[j];
                _cells[i++] = cells2[j];
                j++;
            }
        }

        internal void Fire()
        {
            _cells[_random.Next(CELL_COUNT)].Hit();
        }
    }
}