﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

namespace SudokuMachine
{
    class WindowStuff
    {
        private MachinePad _pad;
        public UIElement Content { get { return _pad.Chrome; } }

        public WindowStuff(Window window)
        {
            _pad = new MachinePad();
            window.KeyDown += new KeyEventHandler(OnKeyDown);

            window.ResizeMode = ResizeMode.CanMinimize;
            window.SizeToContent = SizeToContent.WidthAndHeight;
        }

        private void OnKeyDown(object sender, KeyEventArgs e)
        {
            switch (e.Key)
            {
                case Key.Space:
                    _pad.PushButton1();
                    break;

                case Key.Enter:
                    _pad.PushButton2();
                    break;

                case Key.Escape:
                    _pad.PushButton3();
                    break;

                case Key.X:
                    _pad.PushButton4();
                    break;
            }
        }
    }

    class MachinePad
    {
        private StatusBar _bar;
        private Dashboard _dash;

        private Border _border;
        public Decorator Chrome { get { return _border; } }

        public MachinePad()
        {
            _bar = new StatusBar();
            _dash = new Dashboard(_bar);

            var layout = new StackPanel();
            layout.Children.Add(_bar.Layout);
            layout.Children.Add(_dash.Layout);

            _border = new Border
            {
                Child = layout,
                Padding = new Thickness(MatrixBase.Margin),
            };

            Reset();
        }

        private void Reset()
        {
            _bar.Reset();
            _dash.Reset();
        }

        public void PushButton1()
        {
            _dash.Kickdown1();
        }

        public void PushButton2()
        {
            _dash.Kickdown2();
        }

        public void PushButton3()
        {
            Reset();
        }

        public void PushButton4()
        {
            _dash.Shake();
        }
    }
}